#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "aes.h"

#define N_ITER	    1000000
#define BLOCKLEN    16

int main(void)
{
    const unsigned char raw_key[32] = {
	0x12, 0x34, 0x56, 0x78, 0x9a, 0xbc, 0xde, 0xf0,
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
        0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff,
        0x00, 0xff, 0x0f, 0x33, 0x55, 0xaa, 0xcc, 0xf0 };
    const int       key_len = 32;
    AES_KEY	    key;
    struct timeval  tstart, tend;
    int	            elapsedms;
    unsigned char   buf[BLOCKLEN], buf2[BLOCKLEN];
    int	            i;
    
    printf("\nAES benchmark\n");
    printf("encrypt %d times ...\n", N_ITER);

    gettimeofday(&tstart, NULL);

    AES_set_encrypt_key(raw_key, 8 * key_len, &key);

    for (i = 0; i < BLOCKLEN; i++)
	buf[i] = i;

    for (i = 0; i < N_ITER; i++) {
	AES_encrypt(buf, buf2, &key);
	memcpy(buf, buf2, BLOCKLEN);
    }

    gettimeofday(&tend, NULL);

    for (i = 0; i < BLOCKLEN; i++)
	printf("%02x", buf[i]);
    printf("\n");

    elapsedms = (tend.tv_sec - tstart.tv_sec) * 1000 +
                tend.tv_usec / 1000 - tstart.tv_usec / 1000;
    printf("time elapsed = %d.%03d\n", elapsedms / 1000, elapsedms % 1000);

    return 0;
}

/* vim: noexpandtab softtabstop=4
*/
