#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <Eigen/Core>
#include <Eigen/LU>

USING_PART_OF_NAMESPACE_EIGEN


#define MSIZE   10
#define N_ITER  10000


int main(void)
{
    struct timeval tstart, tend;
    int elapsedms;

    srand(1);

    printf("\nLinear algebra with Eigen\n");
    printf("%d times %dx%d LU decomposition\n", N_ITER, MSIZE, MSIZE);
    gettimeofday(&tstart, NULL);

    MatrixXd mat(MSIZE, MSIZE);
    for (int i = 0; i < MSIZE; i++)
        for (int j = 0; j < MSIZE; j++)
            mat(i, j) = (double)rand() / (RAND_MAX / 10) - 3.0;

    VectorXd vec(MSIZE);
    for (int i = 0; i < MSIZE; i++)
        vec[i] = (double)rand() / (RAND_MAX / 10) - 3.0;

    VectorXd ans;
    for (int i = 0; i < N_ITER; i++) {
        mat.lu().solve(vec, &ans);
        vec[0] += 0.001 * ans[0];
    }

    gettimeofday(&tend, NULL);

    printf("ans = [ ");
    for (int i = 0; i < MSIZE; i++)
        printf("%f ", ans[i]);
    printf("\n");

    elapsedms = (tend.tv_sec - tstart.tv_sec) * 1000 +
                tend.tv_usec / 1000 - tstart.tv_usec / 1000;
    printf("time elapsed = %d.%03d s\n", elapsedms / 1000, elapsedms % 1000);

    return 0;
}
