#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <sys/time.h>
#include "zlib.h"

#define MAX_ZIP_LEN	32000
#define MAX_PLAIN_LEN	128000
#define LOOP_COUNT	80

extern const unsigned char input_data[];
extern const unsigned int input_len;

unsigned char zip_data[MAX_ZIP_LEN];
unsigned char plain_data[MAX_PLAIN_LEN];
unsigned int zip_len;
unsigned int plain_len;

int main(void)
{
	struct timeval tstart, tend;
	const unsigned char *d = input_data;
	unsigned int n = input_len;
	int i, ret, elapsedms;

	printf("\nzlib benchmark\n");
	gettimeofday(&tstart, NULL);

	for (i = 0; i < LOOP_COUNT; i++) {
		z_stream strm;

		memset(plain_data, 0, MAX_PLAIN_LEN);
		plain_len = 0;

		strm.zalloc = Z_NULL;
		strm.zfree  = Z_NULL;
		strm.opaque = Z_NULL;
		strm.avail_in = n;
		strm.next_in  = d;
		ret = inflateInit2(&strm, 0x2f);
		assert(ret == Z_OK);
		strm.avail_out = MAX_PLAIN_LEN;
		strm.next_out  = plain_data;
		ret = inflate(&strm, Z_NO_FLUSH);
		assert(ret == Z_STREAM_END);
		assert(strm.avail_out > 0);
		assert(strm.avail_in == 0);
		plain_len = MAX_PLAIN_LEN - strm.avail_out;
		inflateEnd(&strm);

		memset(zip_data, 0, MAX_ZIP_LEN);

		strm.zalloc = Z_NULL;
		strm.zfree  = Z_NULL;
		strm.opaque = Z_NULL;
		strm.avail_in = 0;
		strm.next_in  = Z_NULL;
		ret = deflateInit(&strm, 6);
		assert(ret == Z_OK);
		strm.avail_in = plain_len;
		strm.next_in  = plain_data;
		strm.avail_out = MAX_ZIP_LEN;
		strm.next_out = zip_data;
		ret = deflate(&strm, Z_FINISH);
		assert(ret == Z_STREAM_END);
		assert(strm.avail_out > 0);
		assert(strm.avail_in == 0);
		zip_len = MAX_ZIP_LEN - strm.avail_out;
		deflateEnd(&strm);

		d = zip_data;
		n = zip_len;
	}

	gettimeofday(&tend, NULL);
	elapsedms = (tend.tv_sec - tstart.tv_sec) * 1000 +
	             tend.tv_usec / 1000 - tstart.tv_usec / 1000;
	printf("done, time elapsed = %d.%03u s\n\n", elapsedms/1000, elapsedms%1000);

	return 0;
}
